P256_DID_PREFIX = b'\x80\x24'
SECP256K1_DID_PREFIX = b'\xe7\x01'

BASE58_MULTIBASE_PREFIX = 'z'
DID_KEY_PREFIX = 'did:key:'

P256_JWT_ALG = 'ES256'
SECP256K1_JWT_ALG = 'ES256K'

P256_CURVE_ORDER = 0xFFFFFFFF_00000000_FFFFFFFF_FFFFFFFF_BCE6FAAD_A7179E84_F3B9CAC2_FC632551
SECP256K1_CURVE_ORDER = 0xFFFFFFFF_FFFFFFFF_FFFFFFFF_FFFFFFFE_BAAEDCE6_AF48A03B_BFD25E8C_D0364141
